// -*- C++ -*-
//
//  This file is part of Siena, a wide-area event notification system.
//  See http://www.cs.colorado.edu/serl/siena/
//
//  Authors: See the file AUTHORS for full details.
//
//  Copyright (C) 2002-2004 University of Colorado
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,
//  USA, or send email to serl@cs.colorado.edu.
//
// $Id: basics.cc,v 1.2 2008-08-21 15:34:10 carzanig Exp $
//
#include <cassert>
#include <iostream>
#include <vector>
#include "siena/simplify.h"
#include "siena/ssimp_types.h"
#include "siena_simplify_impl.h"

using namespace siena_simplify_impl;
using namespace siena;
using namespace std;

int main( int argc, char** argv )
{
  { /* string_t operators */
    vector<string> s;
    vector<string_t> st;
    s.push_back( "a short string" );
    s.push_back( "b short string" );
    s.push_back( "a short" );
    s.push_back( "a shorter stri" );
    s.push_back( "a short" );
    s.push_back( "" );
    s.push_back( "the quick brown fox jumped over the lazy dog" );

    int length = s.size();

    for( int i = 0; i < length; ++i )
      {
	st.push_back( string_t( s[i].c_str(), s[i].length() ) );
      }

    for( int i = 0; i < length; ++i )
      {
	for( int j = 0; j < length; ++j )
	  {
 	    assert( (st[i] < st[j]) == (s[i] < s[j]) );
	    assert( (leq( st[i], st[j])) == (s[i] <= s[j]) );
	    assert( (gt( st[i], st[j])) == (s[i] > s[j]) );
	    assert( (geq( st[i], st[j])) == (s[i] >= s[j]) );
	    assert( (st[i] == st[j]) == (s[i] == s[j]) );
	  }
      }

    string s1( "bytesize" ); string_t s1t( s1.c_str(), s1.length() );
    string s2( "by" ); string_t s2t( s2.c_str(), s2.length() );
    string s3( "ze" ); string_t s3t( s3.c_str(), s3.length() );
    string s4( "e" ); string_t s4t( s4.c_str(), s4.length() );
    string_t et;

    assert( pf( s2t, s1t ) );
    assert( sf( s3t, s1t ) );
    assert( !pf( s1t, s2t ) );
    assert( !sf( s1t, s3t ) );
    assert( !pf( et, s1t ) );
    assert( !pf( et, s2t ) );
    assert( !pf( et, s3t ) );
    assert( !sf( et, s1t ) );
    assert( !sf( et, s2t ) );
    assert( !sf( et, s3t ) );
    assert( !pf( s2t, et ) );
    assert( !sf( s2t, et ) );

    assert( ss( s2t, s1t ) );
    assert( ss( s3t, s1t ) );
    assert( ss( s4t, s1t ) );
    assert( !ss( s1t, s2t ) );
    assert( !ss( s1t, s3t ) );
    assert( !ss( s1t, s4t ) );
    assert( !ss( et, s1t ) );
    assert( !ss( s1t, et ) );
    assert( !ss( et, et ) );
  }
  { /* value operators */
    Constraint iv1( string( "?" ), eq_id, (int_t)10 );
    assert( iv1.bytesize() == 1 + sizeof( operator_id ) + sizeof( type_id ) + sizeof( int ) );

    Constraint iv2( string( "?" ), eq_id, (int_t)11 );
    assert( iv2.bytesize() == 1 + sizeof( operator_id ) + sizeof( type_id ) + sizeof( int ) );

    Constraint iv3( string( "?" ), eq_id, (int_t)11 );
    assert( iv3.bytesize() == 1 + sizeof( operator_id ) + sizeof( type_id ) + sizeof( int ) );

    Constraint dv1( string( "?" ), eq_id, 10.1229 );
    assert( dv1.bytesize() == 1 + sizeof( operator_id ) + sizeof( type_id ) + sizeof( double ) );

    Constraint dv2( string( "?" ), eq_id, 11.99001 );
    assert( dv2.bytesize() == 1 + sizeof( operator_id ) + sizeof( type_id ) + sizeof( double ) );

    Constraint dv3( string( "?" ), eq_id, 11.99001 );
    assert( dv3.bytesize() == 1 + sizeof( operator_id ) + sizeof( type_id ) + sizeof( double ) );

    Constraint bv1( string( "?" ), eq_id, false );
    assert( bv1.bytesize() == 1 + sizeof( operator_id ) + sizeof( type_id ) + sizeof( bool ) );

    Constraint bv2( string( "?" ), eq_id, true );
    assert( bv2.bytesize() == 1 + sizeof( operator_id ) + sizeof( type_id ) + sizeof( bool ) );

    Constraint bv3( string( "?" ), eq_id, true );
    assert( bv3.bytesize() == 1 + sizeof( operator_id ) + sizeof( type_id ) + sizeof( bool ) );

    Constraint sv1( string( "?" ), eq_id, string( "junkString" ) );
    assert( sv1.bytesize() == 1 + sizeof( operator_id ) + sizeof( type_id ) + 10 );

    Constraint sv2( string( "?" ), eq_id, string( "myOtherString" ) );
    assert( sv2.bytesize() == 1 + sizeof( operator_id ) + sizeof( type_id ) + 13 );

    Constraint sv3( string( "?" ), eq_id, string( "myOtherString" ) );
    assert( sv3.bytesize() == 1 + sizeof( operator_id ) + sizeof( type_id ) + 13 );

    /*
     * LESS THAN
     */
    assert( lt( iv1, iv2 ) );
    assert( !lt( iv2, iv1 ) );
    assert( !lt( iv2, iv3 ) );

    assert( lt( dv1, dv2 ) );
    assert( !lt( dv2, dv1 ) );
    assert( !lt( dv2, dv3 ) );

    assert( lt( bv1, bv2 ) );
    assert( !lt( bv2, bv1 ) );
    assert( !lt( bv2, bv3 ) );

    assert( lt( sv1, sv2 ) );
    assert( !lt( sv2, sv1 ) );
    assert( !lt( sv2, sv3 ) );

    /*
     * LESS THAN EQUAL
     */
    assert( leq( iv1, iv2 ) );
    assert( !leq( iv2, iv1 ) );
    assert( leq( iv2, iv3 ) );

    assert( leq( dv1, dv2 ) );
    assert( !leq( dv2, dv1 ) );
    assert( leq( dv2, dv3 ) );

    assert( leq( bv1, bv2 ) );
    assert( !leq( bv2, bv1 ) );
    assert( leq( bv2, bv3 ) );

    assert( leq( sv1, sv2 ) );
    assert( !leq( sv2, sv1 ) );
    assert( leq( sv2, sv3 ) );

    /*
     * GREATER THAN
     */
    assert( !gt( iv1, iv2 ) );
    assert( gt( iv2, iv1 ) );
    assert( !gt( iv2, iv3 ) );

    assert( !gt( dv1, dv2 ) );
    assert( gt( dv2, dv1 ) );
    assert( !gt( dv2, dv3 ) );

    assert( !gt( bv1, bv2 ) );
    assert( gt( bv2, bv1 ) );
    assert( !gt( bv2, bv3 ) );

    assert( !gt( sv1, sv2 ) );
    assert( gt( sv2, sv1 ) );
    assert( !gt( sv2, sv3 ) );

    /*
     * GREATER THAN EQUAL
     */
    assert( !geq( iv1, iv2 ) );
    assert( geq( iv2, iv1 ) );
    assert( geq( iv2, iv3 ) );

    assert( !geq( dv1, dv2 ) );
    assert( geq( dv2, dv1 ) );
    assert( geq( dv2, dv3 ) );

    assert( !geq( bv1, bv2 ) );
    assert( geq( bv2, bv1 ) );
    assert( geq( bv2, bv3 ) );

    assert( !geq( sv1, sv2 ) );
    assert( geq( sv2, sv1 ) );
    assert( geq( sv2, sv3 ) );

    /*
     * NOT EQUAL
     */
    assert( neq( iv1, iv2 ) );
    assert( neq( iv2, iv1 ) );
    assert( !neq( iv2, iv3 ) );

    assert( neq( dv1, dv2 ) );
    assert( neq( dv2, dv1 ) );
    assert( !neq( dv2, dv3 ) );

    assert( neq( bv1, bv2 ) );
    assert( neq( bv2, bv1 ) );
    assert( !neq( bv2, bv3 ) );

    assert( neq( sv1, sv2 ) );
    assert( neq( sv2, sv1 ) );
    assert( !neq( sv2, sv3 ) );
  }
}
